@@ -66,14 +66,6 @@ class Template(object):
 
     Notes:
 
-      * A template refence is a dictionary value string containing ".$.". It
-        has the format::
-
-        "<template reference>.$.<attribute from template's content>".
-
-      * The references can be given to the constructor or at render time. If
-        both are given the render version is used.
-
       * In host2.render(...) above the reference 'host' was used as an alias to
         'host1'.
 
@@ -92,13 +84,15 @@ class Template(object):
 
         """
         self.name = name
+
         if type(content) != types.DictType:
             raise TemplateError("The content given is not a Dict!")
+
         self.content = content
         self.references = references
 
 
-    def render(self, references={}, extendwith={}):
+    def render(self, references={}):
         """Generate a data dict from this template and any it references.
 
         :param references: this is a dict of string to template mappings.
@@ -106,28 +100,23 @@ class Template(object):
         This is used to resolve references to other templates. If this is empty
         self.references will be used instead.
 
-
-        :param extendwith: This is the template to render and then add to this one.
-
-        This will use the rendered dict's update(). This template will overwrite
-        any common keys in the rendered extendwith.
-
-
         :returns: This returns a 'rendered' dict.
 
         All references  will have been replaced with the value the point at.
 
         """
-        return utils.render(
-            self.content.items(),
+        state = utils.RenderState(
+            self,
             int_refs=self.references,
             ext_refs=references,
-            extendwith=extendwith,
+            name=self.name
         )
 
+        return utils.render(state)
+
 
     def items(self):
-        """Used in an all-inclusion render to return our contained content dict.
+        """Used in an all-inclusion / render to return our contained content dict.
         """
         return self.content.items()
 
